@@ -187,7 +187,11 @@ sub reinvite {
 		my ($endpoint,$ctx,$errno,$code,$packet,$leg,$from,$ack) = @_;
 
 		if ( $errno ) {
-			$self->error( "Failed with error $errno".( $code ? " code=$code" :"" ) );
+			if ( $code == 487 ) {
+				# request was canceled, probably be me -> ignore
+			} else {
+				$self->error( "Failed with error $errno".( $code ? " code=$code" :"" ) );
+			}
 			invoke_callback( $param->{cb_final}, 'FAIL',$self,errno => $errno,code => $code,packet => $packet );
 			return;
 		}
@@ -239,11 +243,11 @@ sub reinvite {
 		# This callback will be called on timeout or response to cancel which
 		# got send after ring_time was over
 		my $noanswercb;
-		if ( $param->{ring_time} ) { 
+		if ( $param->{ring_time} ) {
 			$noanswercb = sub {
 				my Net::SIP::Simple::Call $self = shift || return;
 				my ($endpoint,$ctx,$errno,$code,$packet,$leg,$from,$ack) = @_;
-				
+
 				$stopvar = 'NOANSWER' ;
 				my $param = $self->{param};
 				invoke_callback( $param->{cb_noanswer}, 'NOANSWER',$self,
@@ -260,7 +264,7 @@ sub reinvite {
 			# wait until final response
 			$self->loop( $param->{ring_time}, \$stopvar );
 
-			unless ($stopvar) { # timed out 
+			unless ($stopvar) { # timed out
 				$self->{endpoint}->cancel_invite( $self->{ctx},undef, $noanswercb );
 				$self->loop( \$stopvar );
 			}
@@ -275,6 +279,45 @@ sub reinvite {
 }
 
 
+###########################################################################
+# cancel call
+# Args: ($self,%args)
+#   %args:
+#     cb_final: callback when CANCEL was delivered. If not given send_cancel
+#        callback on Call object will be used
+# Returns: true if call could be canceled
+# Comment: cb_final gets triggered if the reply for the CANCEL is received
+# or waiting for the reply timed out
+###########################################################################
+sub cancel {
+	my Net::SIP::Simple::Call $self = shift;
+	my %args = @_;
+
+	my $cb = delete $args{cb_final};
+	%args = ( %{ $self->{param} }, %args );
+	$cb ||= $args{send_cancel};
+
+	my $cancel_cb = [
+		sub {
+			my Net::SIP::Simple::Call $self = shift || return;
+			my ($cb,$args,$endpoint,$ctx,$error,$code) = @_;
+			# we don't care about the cause of this callback
+			# it might be a successful or failed reply packet or no reply
+			# packet at all (timeout) - the call is considered closed
+			# in any case except for 1xx responses
+			if ( $code && $code =~m{^1\d\d} ) {
+				DEBUG( 10,"got prelimary response for CANCEL" );
+				return;
+			}
+			invoke_callback( $cb,$args );
+		},
+		$self,$cb,\%args
+	];
+	weaken( $cancel_cb->[1] );
+
+	return $self->{endpoint}->cancel_invite( $self->{ctx}, undef, $cancel_cb );
+}
+
 ###########################################################################
 # end call
 # Args: ($self,%args)
@@ -483,7 +526,7 @@ sub _setup_local_rtp_socks {
 				proto => 'RTP/AVP',
 				media => 'audio',
 				fmt   => $rp->[0] || 0, # PCMU/8000
-				$rp->[3] ? ( 
+				$rp->[3] ? (
 					a => [ "rtpmap:$rp->[0] $rp->[3]" , "ptime:".$rp->[2]*1000 ]
 				) :(),
 			}
